import optparse, sys, os
import oebakery
from oebakery.cmd_update import UpdateCommand

class CloneCommand:

    def __init__(self, argv):

        parser = optparse.OptionParser("""Usage: oe clone [options]* <repository> [directory]

  Clone OE Bakery development environment into a new directory.

Arguments:
  file        bakery configuration file (remote URL or local file)
  directory   directory to clone into (default is current directory)""")

        (options, args) = parser.parse_args(argv)

        if len(args) < 1:
            parser.error('too few arguments')
        if len(args) > 2:
            parser.error('too many arguments')

        self.repository = args[0]

        if len(args) == 2:
            self.directory = args[1]
        else:
            self.directory = os.path.basename(self.repository)
            if self.directory[-4:] == '.git':
                self.directory = self.directory[:-4]

        self.options = options

        return


    def run(self):

        if not oebakery.call('git clone %s %s'%(self.repository,
                                                self.directory)):
            return

        topdir = oebakery.set_topdir(self.directory)
        oebakery.chdir(self.directory)

        if not oebakery.call('git config push.default tracking'):
            print 'Failed to set push.default = tracking'

        config = oebakery.read_config()

        oebakery.copy_local_conf_sample(config.get('bitbake', 'confdir'))

        self.update_cmd = UpdateCommand(config)
        return self.update_cmd.run()




from __future__ import with_statement # This isn't required in Python 2.6

def get_simple_config_line(filename, variable):
    if os.path.exists(filename):
        regex = re.compile(variable +'\s*=\s*[\"\'](.*)[\"\']')
        with open(filename) as file:
            for line in file.readlines():
                match = regex.match(line)
                if match:
                    return match.group(1)
    return None


def version_str_to_tuple(str):
    dash = str.rfind('-')
    if dash >= 0:
        str = str[dash+1:]
    list = string.split(str, '.')
    tuple = ()
    for number in list:
        tuple = tuple + (int(number),)
    return tuple



#!/usr/bin/env python
import sys, dircache, subprocess, os, string, re, glob, hashlib

def main():

    usage="""Usage: oe <command> [options]*

Allowed oe commands are:
  init        Setup new OE Bakery development environment
  clone       Clone an OE Bakery development environment into a new directory
  update      Update OE Bakery development environment
  tmp         Manage TMPDIR directories
  bake        Build recipe (call bitbake)
  ingredient  Manage ingredient (downloaded sources) files
  prebake     Manage prebake (packaged staging) files

See 'oe <command> -h' or 'oe help <command> for more information on a
specific command."""

    if len(sys.argv) < 2 or (len(sys.argv) == 2 and sys.argv[1] == '-h'):
        print usage
        return

    if sys.argv[1] == "help":
        if len(sys.argv) == 3:
            sys.argv[1] = sys.argv[2]
            sys.argv[2] = "-h"
        else:
            print usage
            return

    # hack to be able to run from source directory
    sys.path.insert(
        0, os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

    import oebakery
    from oebakery.cmd_init import InitCommand
    from oebakery.cmd_clone import CloneCommand
    from oebakery.cmd_update import UpdateCommand
    from oebakery.cmd_tmp import TmpCommand
    from oebakery.cmd_bake import BakeCommand
    #from oebakery.cmd_ingredient import IngredientCommand
    #from oebakery.cmd_prebake import PrebakeCommand
    from oebakery import misc

    if sys.argv[1] == "init":
        cmd = InitCommand(sys.argv[2:])
        return cmd.run()

    elif sys.argv[1] == "clone":
        cmd = CloneCommand(sys.argv[2:])
        return cmd.run()

    topdir = oebakery.locate_topdir()
    if topdir != os.environ['PWD']:
        oebakery.chdir(topdir)

    config = oebakery.read_config()

    if sys.argv[1] == "update":
        cmd = UpdateCommand(config, sys.argv[2:])

    elif sys.argv[1] == "tmp":
        cmd = TmpCommand(config, sys.argv[2:])

    elif sys.argv[1] == "bake":
        cmd = BakeCommand(config, sys.argv[2:])

    elif sys.argv[1] == "ingredient":
        cmd = IngredientCommand(config, sys.argv[2:])

    elif sys.argv[1] == "prebake":
        cmd = PrebakeCommand(config, sys.argv[2:])

    else:
        print usage
        sys.exit(1)

    return cmd.run()
    

if __name__ == "__main__":
    main()



from __future__ import with_statement # This isn't required in Python 2.6
import oebakery, optparse, ConfigParser, os, re, sys
from oebakery.cmd_tmp import TmpCommand

class BakeCommand:

    def __init__(self, config, argv):

        parser = optparse.OptionParser("""Usage: oe bake [bitbake options]

  Build recipes using BitBake in the current OpenEmbedded development
  environment.  This can be done from any subdirectory of the OpenEmbedded
  development environment.""")

        self.args = argv
        self.config = config

        return


    def run(self):
    
        topdir = oebakery.get_topdir()

        self.tmp_cmd = TmpCommand(self.config)
        tmpdir = self.tmp_cmd.get_tmpdir()

        os.environ['OE_HOME'] = topdir
        os.environ['OE_TMPDIR'] = tmpdir
        os.environ['PATH'] = '%s:%s'%(
            os.path.join(topdir, self.config.get('bitbake', 'path')),
            os.environ['PATH'])
        bbpath = ''
        for path in self.config.get('bitbake', 'bbpath').split(':'):
            bbpath += os.path.normpath(os.path.join(topdir, path)) + ':'
        bbpath = bbpath.strip(':')
        os.environ['BBPATH'] = bbpath

        os.environ['BB_ENV_EXTRAWHITE'] = 'OE_HOME OE_TMPDIR'

        print 'OE_HOME        =', os.environ['OE_HOME']
        print 'OE_TMPDIR      =', os.environ['OE_TMPDIR']
        print 'BBPATH         =', os.environ['BBPATH']
        print ''

        oebakery.call(['bitbake'] + self.args)

        return



from __future__ import with_statement # This isn't required in Python 2.6

def do_mirror(argv):
    usage="""Usage oe mirror <sub-command> [options]

Allowed mirror sub-commands are:
  put      Upload local download files to download mirror
  get      Download files from download mirror
  cmp      Compare local download files against download mirror copies
  save     Save list of local download files to download mirror
  load     Load missing files from list of download files on download mirror

See 'oe mirror <sub-command> -h' for more information on a specific sub-command."""

    if len(argv) < 1:
        print usage
        return

    if argv[0] == "help":
        if len(argv) == 1:
            print usage
            return
        elif len(argv) == 2:
            argv[0] = argv[1]
            sys.argv[1] = "-h"

    if argv[0] == "put":
        do_mirror_put(argv[1:])
    elif argv[0] == "get":
        do_mirror_get(argv[1:])
    elif argv[0] == "cmp":
        do_mirror_cmp(argv[1:])
    elif argv[0] == "save":
        do_mirror_save(argv[1:])
    elif argv[0] == "load":
        do_mirror_load(argv[1:])
    else:
        print usage
        sys.exit(1)


def do_mirror_put(argv):

    parser = OptionParser("""Usage: oe mirror put [file]*

  Upload file(s) from downloads directory to download mirror.""")
    parser.add_option("-c", "--check-md5",
                      action="store_true", dest="check_md5", default=False,
                      help="verify MD5 sums")
    parser.add_option("-i", "--interactive",
                      action="store_true", dest="interactive", default=False,
                      help="interactive mode")
    parser.add_option("-n", "--dry-run",
                      action="store_true", dest="dry_run", default=False,
                      help="perform a trial run with no changes made")
    (options, args) = parser.parse_args(argv)

    if len(args) == 0:
        args = ['*']

    topdir = cd_topdir()
    print "OpenEmbedded Bakery",topdir

    config = read_config()

    if not config.has_section("mirror"):
        print >>sys.stderr, "ERROR: no [mirror] section in conf/oe.conf"
        return
    if not config.has_option("mirror", "put_hostname"):
        print >>sys.stderr, "ERROR: no put_hostname option in conf/oe.conf"
        return
    if not config.has_option("mirror", "put_path"):
        print >>sys.stderr, "ERROR: no put_path option in conf/oe.conf"
        return

    os.chdir('downloads')

    local_files = ls_local_files(args, options)

    if len(local_files) == 0:
        print "No files to upload"
        return

    local_files = mirror_files_filter(config, options, local_files)

    if len(local_files) == 0:
        print "No files to upload"
        return

    md5_files = []
    for file in local_files:
        md5_files += [file + '.md5']
    local_files = local_files + md5_files
    local_files.sort()

    local_files_str = ""
    for file in local_files:
        local_files_str = local_files_str + ' ' + file

    call("scp%s %s:%s"%(local_files_str,
                         config.get('mirror', 'put_hostname'),
                         config.get('mirror', 'put_path')),
         dry_run=options.dry_run)

    return


def ls_local_files(args, options):
    local_files = []

    for arg in args:

        files = glob.glob(arg)
        files.sort()

        for file in files:

            if not os.path.isfile(file):
                continue
            if file[-4:] == '.md5':
                continue
            if file[-5:] == '.lock':
                continue
            if file[:4] == 'git_':
                continue
            if file[:4] == 'cvs_':
                continue
            if file[:4] == 'svn_':
                continue

            local_files.append(file)

    filtered_local_files = []
    for file in local_files:
        md5_file = file + '.md5'
        if not os.path.exists(md5_file):
            print "\nWARNING: no MD5 sum for", file
            if options.interactive:
                while True:
                    answer = raw_input(
                        "(s)kip file, (r)emove file, (g)enerate md5: ")
                    if answer == 's':
                        break
                    elif answer == 'r':
                        os.remove(file)
                        break
                    elif answer == 'g':
                        with open(md5_file, "w+") as wfile:
                            wfile.write(calc_md5sum(file))
                        filtered_local_files.append(file)
                        break
                    else:
                        continue

        else:
            filtered_local_files.append(file)
            
    local_files = filtered_local_files

    if options.check_md5:
        filtered_local_files = []
        for file in local_files:
            md5_filename = file + '.md5'
            with open(md5_file, "r") as local_md5_file:
                local_md5 = local_md5_file.readline()
            local_md5_gen = calc_md5sum(file)
            if local_md5_gen == local_md5:
                filtered_local_files.append(file)
            else:
                print "\nWARNING: bad MD5 sum for", file
                if options.interactive:
                    while True:
                        answer = raw_input(
                            "(s)kip file, (r)emove file, (g)enerate new md5 file: ")
                        if answer == 's':
                            break
                        elif answer == 'r':
                            os.remove(file)
                            break
                        elif answer == 'g':
                            with open(file+'.md5', "w+") as wfile:
                                wfile.write(local_md5_gen)
                                filtered_local_files.append(file)
                            break
                        else:
                            continue
                else:
                    continue
        local_files = filtered_local_files

    return local_files



def ls_mirror_files(config):
    pipe = subprocess.Popen(
        ['ssh', config.get('mirror', 'put_hostname'),
         'ls', config.get('mirror', 'put_path')],
        stdout=subprocess.PIPE)
    
    mirror_files = []
    for file in pipe.stdout.readlines():
        file = file.rstrip()
        mirror_files.append(file)

    return mirror_files


def mirror_files_filter(config, options, local_files):

    mirror_files = ls_mirror_files(config)

    mirror_files = remove_not_in_local_files(mirror_files, local_files)

    (mirror_files, local_files) = filter_mirror_files_md5_sanity(
        mirror_files, local_files, options)
    
    if options.check_md5:
        local_files = filter_compare_md5(local_files, mirror_files, config,
                                         options)
    else:
        local_files = remove_in_mirror_files(local_files, mirror_files)

    return local_files

def remove_not_in_local_files(mirror_files, local_files):
    filtered_mirror_files = []
    for file in mirror_files:
        if file[-4:] == '.md5':
            if file[:-4] in local_files:
                filtered_mirror_files.append(file)
        else:
            if file in local_files:
                filtered_mirror_files.append(file)
    return filtered_mirror_files


def filter_mirror_files_md5_sanity(mirror_files, local_files, options):
    filtered_mirror_files = []
    for file in mirror_files:
        if file[-4:] == '.md5':
            if not file[:-4] in mirror_files:
                print '\nWARNING: mirror has orphan MD5 sum for', file
            else:
                filtered_mirror_files.append(file)
        else:
            filtered_mirror_files.append(file)

    filtered_local_files = []
    for file in local_files:
        if (file in mirror_files) and (not file + '.md5' in mirror_files):
            print '\nWARNING: mirror missing MD5 sum for', file
            if options.interactive:
                while True:
                    answer = raw_input(
                        "(s)kip file, (o)verwrite mirror copy: ")
                    if answer == 's':
                        break
                    elif answer == 'o':
                        filtered_local_files.append(file)
                        break
                    else:
                        continue
        else:
            filtered_local_files.append(file)

    return (filtered_mirror_files, filtered_local_files)


def filter_compare_md5(local_files, mirror_files, config, options):

    filtered_local_files = []
    for file in local_files:
        md5_file = file + '.md5'
        if not md5_file in mirror_files:
            filtered_local_files.append(file)
        else:
            mirror_md5 = subprocess.Popen(
                ['ssh', config.get('mirror', 'put_hostname'),
                 'cat',
                 os.path.join(config.get('mirror', 'put_path'), md5_file)],
                stdout=subprocess.PIPE).communicate()[0]
            with open(md5_file, "r") as local_md5_file:
                local_md5 = local_md5_file.readline()
            if local_md5 == mirror_md5:
                continue
            else:
                print "\nWARNING: md5 mismatch for %s"%file
                print 'local md5  =', local_md5
                print 'mirror md5 =', mirror_md5
                if options.interactive:
                    while True:
                        answer = raw_input(
                            "(s)kip file, (o)verwrite mirror copy, (g)et mirror copy: ")
                        if answer == 's':
                            break
                        elif answer == 'o':
                            filtered_local_files.append(file)
                            break
                        elif answer == 'g':
                            print "WARNING: get not implemented"
                            break
                        else:
                            continue
    return filtered_local_files


def remove_in_mirror_files(local_files, mirror_files):
    filtered_local_files = []
    for file in local_files:
        if not file in mirror_files:
            filtered_local_files.append(file)
    return filtered_local_files


def calc_md5sum(filename):
    m = hashlib.md5()
    with open(filename, "r") as file:
        while True:
            bytes = file.read(4096)
            if len(bytes) == 0:
                break
            m.update(bytes)
    return m.hexdigest()



from __future__ import with_statement # This isn't required in Python 2.6

__version__ = '0.5'

__all__ = [

    'set_topdir',
    'locate_topdir',
    'get_topdir',
    'read_config',
    'call',

]

import sys, os, subprocess, re, ConfigParser, string, shutil


TOPDIR = None

def set_topdir(dir):
    global TOPDIR

    if not (os.path.exists(os.path.join(dir, 'bakery.ini'))
            or os.path.exists(os.path.join('.bakery'))):
        print >>sys.stderr, 'ERROR: not a valid OE Bakery development environment:', dir
        return None

    TOPDIR = os.path.abspath(dir)

    return TOPDIR
    

def locate_topdir():
    global TOPDIR

    TOPDIR = locate_topdir_recursive(os.getenv('PWD'))

    if not TOPDIR:
        print >>sys.stderr, 'ERROR: current directory is not part of an OE Bakery development environment'
        sys.exit(1)

    return TOPDIR


def locate_topdir_recursive(dir):

    if dir == '/':
        return None

    if (os.path.exists('%s/bakery.ini'%dir) or os.path.exists('%s/.bakery'%dir)):
        return os.path.abspath(dir)

    return locate_topdir_recursive(os.path.dirname(dir))


def get_topdir():
    global TOPDIR
    return TOPDIR


def read_config():
    config = ConfigParser.SafeConfigParser()

    if os.path.exists('bakery.ini'):
        inifile = 'bakery.ini'
    elif os.path.exists('.bakery'):
        inifile = '.bakery'
    else:
        print >>sys.stderr, 'ERROR: no bakery.ini or .bakery in current directory'
        sys.exit(1)

    if not config.read(inifile):
        print >>sys.stderr, 'ERROR: failed to read %s'%inifile
        sys.exit(1)

    if not config.has_section('tmp'):
        config.add_section('tmp')
    if not config.has_option('tmp', 'tmpdir'):
        config.set('tmp', 'tmpdir', 'tmp')
    if not config.has_option('tmp', 'default'):
        config.set('tmp', 'default', 'default')

    if not config.has_section('bitbake'):
        config.add_section('bitbake')
    if not config.has_option('bitbake', 'path'):
        config.set('bitbake', 'path', 'bitbake/bin')
    if not config.has_option('bitbake', 'bbpath'):
        config.set('bitbake', 'bbpath', '')
    if not config.has_option('bitbake', 'confdir'):
        config.set('bitbake', 'confdir', 'conf')

    return config


def get_simple_config_line(filename, variable):
    if os.path.exists(filename):
        regex = re.compile(variable +'\s*=\s*[\"\'](.*)[\"\']')
        with open(filename) as file:
            for line in file.readlines():
                match = regex.match(line)
                if match:
                    return match.group(1)
    return None


def call(cmd, dir=None, quiet=False, success_returncode=0):

    if type(cmd) == type([]):
        cmdlist = cmd
        cmd = cmdlist[0]
        for arg in cmdlist[1:]:
            cmd = cmd + ' ' + arg

    if dir:
        pwd = os.getcwd()
        chdir(dir, quiet)

    if not quiet:
        print '>', cmd

    retval = None
    if quiet:
        process = subprocess.Popen(cmd, shell=True, stdin=sys.stdin,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)
        output = process.communicate()[0]
        if process.returncode == success_returncode:
            retval = output

    else:
        returncode = subprocess.call(cmd, shell=True, stdin=sys.stdin)
        if returncode == success_returncode:
            retval = True

    if dir:
        chdir(pwd, quiet)

    return retval


def chdir(dir, quiet=False):
    if os.path.realpath(os.path.normpath(dir)) == os.path.normpath(os.getcwd()):
        return

    if not quiet:
        print '> cd', dir

    os.chdir(dir)

    return


def copy_local_conf_sample(confdir):
    
    config = os.path.join(confdir, 'local.conf')
    sample = os.path.join(confdir, 'local.conf.sample')
    if not os.path.exists(config) and os.path.exists(sample):
        try:
            print '> cp %s %s'%(sample, config)
            shutil.copyfile(sample, config)
        except:
            print 'Warning: failed to write config file:', config


#def fetch_file(file):
#
#    colon = file.find(':')
#
#    if colon == -1:
#        print 'file is local path'
#        return shutil.copyfile(self.file, 'conf/bakery.ini')
#
#    elif file[:colon] in ['http', 'ftp', 'https']:
#        print 'use wget'
#        return call('wget -O conf/bakery.ini %s'%(file))
#
#    elif self.file[:colon] in ['ssh']:
#        print 'use scp (fall through)'
#        file = file[colon+3:]
#
#    elif file[colon+1:colon+3] == '//':
#        print 'invalid url'
#        return
#
#    print 'use scp'
#    bakery.call('scp %s conf/bakery.ini'%(file))
#
#    return



import os, subprocess, socket, sys, string
import optparse

import oebakery

class UpdateCommand:

    def __init__(self, config, argv=[]):

        parser = optparse.OptionParser("""Usage: oe update [options]

  Update OE Bakery development environment in the current directory.""")

        parser.add_option("-p", "--pull",
                          action="store_true", dest="pull", default=False,
                          help="pull from remote repositories")

        (self.options, self.args) = parser.parse_args(argv)

        self.config = config

        return


    def run(self):

        if not os.path.exists('.git'):
            print 'Aiee!  This is not a git repository!!'
            return

        if self.options.pull and not oebakery.call('git pull'):
            print 'Failed to pull updates to main repository'

        if self.config.has_section('remotes'):
            for (name, url) in self.config.items('remotes'):
                git_update_remote(name, url)

        if self.options.pull:
            # older git versions return 1 on success, and newer versions 0
            # so we will just ignore the return code for now
            oebakery.call('git remote update')

        if self.options.pull and self.config.has_section('remotes'):
            for (name, url) in self.config.items('remotes'):
                if not oebakery.call('git remote prune %s'%(name)):
                    print 'Failed to prune remotes for main repository'

        if os.path.exists('.gitmodules'):
            if not oebakery.call('git submodule update --init'):
                print 'Failed to update git submodules'
                return

        if self.config.has_section('submodules'):

            for (path, url) in self.config.items('submodules'):

                if url.find(' ') >= 0:
                    (url, version) = string.rsplit(url, maxsplit=1)
                else:
                    version = None

                section_name = 'remotes "%s"'%path
                if self.config.has_section(section_name):
                    remotes = self.config.items(section_name)
                else:
                    remotes = None

                git_update_submodule(path, url, version, remotes,
                                              self.options.pull)

        return


def git_update_remote(name, url):

    # fetch list of remotes
    remotes_list = oebakery.call('git remote -v', quiet=True).split('\n')
    remotes = {}
    for remote in remotes_list:
        if '\t' in remote:
            (iter_name, iter_url) = string.split(remote, '\t', maxsplit=1)
            remotes[iter_name] = iter_url

    # if matching remote is found, do nothing and return
    if remotes.has_key(name) and remotes[name] == url:
        return

    # if remote is found, but with different url, change url and return
    if remotes.has_key(name):
        oebakery.call('git config remote.%s.url %s'%(name, url))
        return

    # if another remote is found with same url, rename it and return
    	# generally seems like a bad idea, so let's not do that...

    if not oebakery.call('git remote add %s %s'%(name, url)):
        print 'Failed to add remote "%s"'%name
        return

    return


def git_update_submodule(path, url, version=None, remotes=None, pull=False):

    oebakery.chdir(oebakery.get_topdir())

    pristine_clone = False
    if not os.path.exists(os.path.join(path, '.git')):

        # git clone
        if version:
            if not oebakery.call('git clone -n %s %s'%(url, path)):
                print 'Failed to clone submodule %s'%path
                return
            pristine_clone = True
        else:
            if not oebakery.call('git clone %s %s'%(url, path)):
                print 'Failed to clone submodule %s'%path
                return

    oebakery.chdir(path)

    # update origin url if necessary
    current_url = oebakery.call('git config --get remote.origin.url',
                                quiet=True).strip()
    if url != current_url:
        if not oebakery.call('git config remote.origin.url %s'%(url)):
            print 'Failed to set origin url for "%s": %s'%(path, url)

    # fetch updates from origin
    if pull and not oebakery.call('git fetch origin'):
        print 'Failed to fetch updates from origin for "%s"'%path

    if version:

        # check if version is a valid commit or remote branch name
        commit = branch = False
        descr = oebakery.call('git describe --all %s'%version, quiet=True)
        if descr:
            commit = descr
        else:
            branch = 'remotes/origin/%s'%version
            if oebakery.call('git describe --all %s'%branch, quiet=True):
                pass
            else:
                print 'Error: invalid version for submodule %s: %s'%(
                    path, version)
                return
    
        # create and checkout local branch tracking the requested remote branch
        if branch:
            if not oebakery.call('git checkout --track -b %s %s'%(
                    version, branch)):
                print 'Failed to create local tracking branch for', branch
                return
    
        # checkout the requested version (specific commit, or local branch)
        elif (pristine_clone or
              commit != oebakery.call('git describe --all HEAD', quiet=True)):
            if not oebakery.call('git checkout %s'%version):
                print 'Failed to checkout', version
                return

    if remotes and len(remotes) > 0:
        for (name, url) in remotes:
            git_update_remote(name, url)

    # older git versions return 1 on success, and newer versions 0
    # so we will just ignore the return code for now
    oebakery.call('git remote update')

    if pull and remotes and len(remotes) > 0:
        for (name, url) in remotes:
            if not oebakery.call('git remote prune %s'%(name)):
                print 'Failed to prune remotes for %s'%path

    return



from __future__ import with_statement # This isn't required in Python 2.6
import oebakery, optparse, ConfigParser, os, re, sys, shutil

class TmpCommand:

    def __init__(self, config, argv=[]):

        parser = optparse.OptionParser("""Usage: oe tmp [option] [name]*

Manage OpenEmbedded TMPDIR's

  oe tmp                      show available tmp areas
  oe tmp <name>		      select tmp area
  oe tmp -c|--create <name>   create (and select) tmp area
  oe tmp -d|--delete <name>*  delete tmp area(s)""")

        parser.add_option("-c", "--create",
                          action="store_true", dest="create", default=False,
                          help="create new tmp area")

        parser.add_option("-d", "--delete",
                          action="store_true", dest="delete", default=False,
                          help="delete tmp area")

        (self.options, self.args) = parser.parse_args(argv)

        if ((self.options.create and self.options.delete)
            or (self.options.create and len(self.args) != 1)
            or (self.options.delete and len(self.args) < 1)):
            print >>sys.stderr, 'Invalid arguments'
            parser.print_help()
            sys.exit(1)

        for arg in self.args:
            if '/' in arg:
                print >>sys.stderr, 'invalid tmp name:', arg
                sys.exit(1)

        self.config = config

        if self.config.has_option('tmp', 'basedir'):
            self.basedir = os.path.normpath(
                os.path.join(oebakery.get_topdir(),
                             self.config.get('tmp', 'basedir')))
        else:
            self.basedir = None

        self.tmpdir = os.path.normpath(
            os.path.join(oebakery.get_topdir(),
                         self.config.get('tmp', 'tmpdir')))

        return


    def get_tmpdir(self):

        if os.path.exists(self.tmpdir):
            return os.path.realpath(self.tmpdir)

        if self.basedir:
            if not os.path.exists(self.basedir):
                os.makedirs(self.basedir)
            self.create_tmp_area(self.config.get('tmp', 'default'))

        else:
            os.makedirs(self.tmpdir)

        return os.path.realpath(self.tmpdir)


    def run(self):

        if not self.basedir:
            print >>sys.stderr, 'Error: no tmp.basedir specified'
            sys.exit(1)

        if not os.path.exists(self.basedir):
            os.makedirs(self.basedir)

        if self.options.create:
            self.create_tmp_area(self.args[0])

        elif self.options.delete:
            for tmp in self.args:
                self.delete_tmp_area(tmp)
            return

        elif len(self.args) == 0:
            return self.show_tmp_areas()

        else:
            return self.select_tmp_area(self.args[0])


    def create_tmp_area(self, name):

        areadir = os.path.join(self.basedir, name)

        if not os.path.exists(areadir):
            os.makedirs(areadir)

        return self.select_tmp_area(name)


    def select_tmp_area(self, name):

        areadir = os.path.join(self.basedir, name)

        if not os.path.exists(areadir):
            print >>sys.stderr, 'no such directory:', areadir
            print >>sys.stderr, 'you can create it with'
            print >>sys.stderr, '  oe tmp -c', name
            return

        if os.path.exists(self.tmpdir) and not os.path.islink(self.tmpdir):
            print >>sys.stderr, 'not a symlink:', self.tmpdir

        if os.path.islink(self.tmpdir):
            os.remove(self.tmpdir)

        try:
            arealink = os.path.relpath(areadir, os.path.dirname(self.tmpdir))
        except AttributeError:
            arealink = areadir
        os.symlink(arealink, self.tmpdir)

        return


    def delete_tmp_area(self, name):

        areadir = os.path.join(self.basedir, name)

        if not os.path.exists(areadir):
            print 'no such directory:', areadir
            return

        real_tmpdir = os.path.realpath(self.tmpdir)
        real_areadir = os.path.realpath(areadir)
        if real_areadir == real_tmpdir:
            print >>sys.stderr, 'Error: cannot delete active tmp area'
            sys.exit(1)

        if os.path.islink(areadir):
            areadir = os.path.realpath(areadir)

        shutil.rmtree(areadir)

        return


    def show_tmp_areas(self):

        if not self.basedir:
            print >>sys.stderr, 'Error: no tmp.basedir specified'
            sys.exit(1)

        if not os.path.isdir(self.basedir):
            return

        real_tmpdir = os.path.realpath(self.tmpdir)

        for areadir in sorted(os.listdir(self.basedir)):
            real_areadir = os.path.realpath(
                os.path.join(self.basedir, areadir))
            if not os.path.isdir(real_areadir):
                continue
            if real_areadir == real_tmpdir:
                print '*', os.path.basename(areadir)
            else:
                print ' ', os.path.basename(areadir)

        return



import os, subprocess, socket, shutil, optparse
import oebakery
from oebakery.cmd_update import UpdateCommand

class InitCommand:

    def __init__(self, argv=[]):

        parser = optparse.OptionParser("""Usage: oe init [options]

  Setup OE Bakery development environment in the current directory.""")

        (self.options, self.args) = parser.parse_args(argv)

        self.config = oebakery.read_config()

        return


    def run(self):

        topdir = oebakery.set_topdir(os.path.curdir)

        if not os.path.exists('.git'):

            if not oebakery.call('git init'):
                print 'Failed to initialize git'
                return

            if not oebakery.call('git config push.default tracking'):
                print 'Failed to set push.default = tracking'

        oebakery.copy_local_conf_sample(self.config.get('bitbake', 'confdir'))

        self.update_cmd = UpdateCommand(self.config)
        return self.update_cmd.run()



#! /usr/bin/env python
from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages
import sys, os
try:
    import stdeb
except ImportError:
    print 'Warning: cannot find stdeb module'

version = '0.14.2'

setup(name='oe-bakery',
      version=version,
      description="OE-Bakery - OpenEmbedded Development Environment Tool",
      long_description="""\
OE-Bakery - A tool for making working with BitBake and OpenEmbedded easier""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author="Esben Haabendal",
      author_email="eha@doredevelopment.dk",
      license="GPL",
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      url="http://dev.doredevelopment.dk/wiki/OpenEmbeddedBakery",
      entry_points = {
        'console_scripts': [
            'oe = oebakery.oe:main',
            ]
        }
      )



def do_config(argv):

    parser = OptionParser("""Usage: oe config [FILE] [options]

  Configure OpenEmbedded development environment. Choose a local.conf from
  the conf/local.conf.d directory.  If FILE is given, use that. If FILE is
  not given, present user with a menu of available configuration files.""")
    (options, args) = parser.parse_args(argv)

    if len(args) > 1:
        parser.print_help()
        return

    topdir = cd_topdir()

    if len(args) == 0:
        return do_config_menu()

    if len(args) == 1:
        return do_config_file(argv[0])


def do_config_menu():

    if not os.path.isdir("conf/local.conf.d"):
        print >>sys.stderr, "ERROR: conf/local.conf.d directory not found"
        return

    print "Choose configuration file (local.conf will be a symlink to it)\n"
    files = []
    for file in dircache.listdir("conf/local.conf.d"):
        if file[-1:] == '~':
            continue
        files.append(file)
        print " %s: %s"%(("%s"%(len(files))).rjust(3), file)

    answer = raw_input("\nEnter configuration name or number: ")
    try:
        number = int(answer)
        if number > len(files) or number <= 0:
            print >>sys.stderr, "ERROR: invalid number"
            return
        file = files[number-1]
    except:
        if not answer in files:
            print >>sys.stderr, "ERROR: bad configuration name"
            return
        file = answer
        
    return do_config_file(file)


def do_config_file(filename):

    if not os.path.exists("conf/local.conf.d/%s"%filename):
        print >>sys.stderr, "ERROR: %s configuration file not found"%filename
        return

    if (os.path.exists("conf/local.conf") and
        not os.path.islink("conf/local.conf")):
        print >>sys.stderr, "ERROR: conf/local.conf is not a symlink, refusing to remove it"
        return

    if os.path.islink("conf/local.conf"):
        os.remove("conf/local.conf")

    os.symlink("local.conf.d/%s"%filename, "conf/local.conf")

    return
